import 'package:ably_flutter/ably_flutter.dart';
import 'package:meta/meta.dart';

/// Contains the properties of a device registered for push notifications.
@immutable
class DeviceDetails {
  /// A unique ID generated by the device.
  final String? id;

  /// The client ID the device is connected to Ably with.
  final String? clientId;

  /// The [DevicePlatform] associated with the device.
  ///
  /// Describes the platform the device uses, such as `android` or `ios`.
  final DevicePlatform platform;

  /// The [FormFactor] object associated with the device.
  ///
  /// Describes the type of
  /// the device, such as `phone` or `tablet`.
  final FormFactor formFactor;

  /// A [Map] object of key-value pairs that contains metadata for the device.
  final Map<String, String> metadata;

  /// The [DevicePushDetails] object associated with the device.
  ///
  /// Describes the
  /// details of the push registration of the device.
  final DevicePushDetails push;

  /// @nodoc
  /// Initializes an instance without any defaults
  const DeviceDetails({
    required this.formFactor,
    required this.metadata,
    required this.platform,
    required this.push,
    this.clientId,
    this.id,
  });
}
